﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Diagnostics;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel.DomainServices.Client;
using System.Text;
using System.Threading;
using System.Windows;
using RoomManagement.Web.Services;

namespace RoomManagement.Web.Models
{
    // The majority of this is only used in ReportView

    [DebuggerDisplay("{FullNumber}")]
    public partial class Cell
    {
        // If the room is not a desksharing one, this makes everything much simpler
        public Person Person
        {
            get
            {
                return ( People != null && People.Count == 1 ) ? People.First() : null;
            }
            set
            {
                if ( this.CellTypeID == Constants.DeskSharingCellTypeId ) throw new NotSupportedException( "Person cannot be set since this cell is a desk sharing one and thus can have more than one person. Use People instead." );
                People.Remove( Person );
                People.Add( value );
                RaisePropertyChanged( "IsFree" );
            }
        }

        // Those three are to display all the people in a desksharing room

        public string PeopleVisas
        {
            get
            {
                if ( this.IsFree ) return string.Empty;
                if ( this.People.Count == 1 ) return this.Person.Visa;
                string visas = this.People.Aggregate(string.Empty, (current, p) => current + (p.Visa + "\n")); // "Sum" of all visas
                return visas.Remove( visas.Length - 1 ); // remove the last line break;
            }
        }
        public string PeopleFirstNames
        {
            get
            {
                if ( this.IsFree ) return string.Empty;
                if ( this.People.Count == 1 ) return this.Person.FirstName;
                string firstNames = this.People.Aggregate(string.Empty, (current, p) => current + (p.FirstName + "\n")); // "Sum" of all FirstNames
                return firstNames.Remove( firstNames.Length - 1 ); // remove the last line break;
            }
        }
        public string PeopleLastNames
        {
            get
            {
                if ( this.IsFree ) return string.Empty;
                if ( this.People.Count == 1 ) return this.Person.LastName;
                string lastNames = this.People.Aggregate( string.Empty, ( current, p ) => current + ( p.LastName + "\n" ) ); // "Sum" of all LastNames
                return lastNames.Remove( lastNames.Length - 1 ); // remove the last line break;
            }
        }

        public void RaiseAllChanges()
        {
            RaisePropertyChanged( "PeopleVisas" );
            RaisePropertyChanged( "PeopleFirstNames" );
            RaisePropertyChanged( "PeopleLastNames" );
            this.Room.RaiseAllChanges();
        }

        // The constructor is *not* called when an object is deserialized. 
        // And it's not written  anywhere in the method descriptions...
        [OnDeserialized]
        new public void OnDeserialized( StreamingContext value )
        {
            OnCreated();
        }

        partial void OnCreated()
        {
            INotifyCollectionChanged ncc = People;
            ncc.CollectionChanged += delegate { RaisePropertyChanged( "Person" ); RaisePropertyChanged( "IsFree" ); };
        }
        public string FullNumber
        {
            get
            {
                return this.Room.Number + "-" + this.Number;
            }
        }

        public bool IsFree
        {
            get
            {
                // A visitor cell is always free.
                if ( this.CellTypeID == Constants.VisitorCellTypeId ) return true;
                return ( this.People == null || this.People.Count == 0 );
            }
        }
    }

    [DebuggerDisplay("Number = {Number}")]
    public partial class Room
    {
        public int FreeCellsCount
        {
            get
            {
                int count = this.Cells.Where( c => c.Person == null ).Count();
                return count;
            }
        }
        public string FreeCells
        {
            get
            {
                return "(" + FreeCellsCount + " free)";
            }
        }
        public bool HasFreeCells
        {
            get
            {
                return FreeCellsCount != 0;
            }
        }

        public void RaiseAllChanges()
        {
            this.RaisePropertyChanged( "FreeCellsCount" );
            this.RaisePropertyChanged( "FreeCells" );
            this.RaisePropertyChanged( "HasFreeCells" );
            this.Floor.RaiseFreeCellsChanges();
        }

        protected override void ValidateProperty(ValidationContext validationContext, object value) {

            if (validationContext.MemberName == "PhoneNumber") {
                return; 
            }
            base.ValidateProperty(validationContext, value);
        }

    }

    [DebuggerDisplay("Name = {Name}")]
    public partial class Floor
    {
        public static bool IsSite { get { return false; } } // HACK used to display "total" in ReportView. Much simpler than having site implement ICloneable.

        public int CellsCount
        {
            get
            {
                return this.Rooms.Sum( r => r.Cells.Count );
            }
        }

        public int FreeCellsCount
        {
            get
            {
                return this.Rooms.Sum( r => r.Cells.Where( c => c.IsFree ).Count() );
            }
        }
        public string FreeCells
        {
            get
            {
                return "(" + FreeCellsCount + " free)";
            }
        }
        public bool HasFreeCells
        {
            get
            {
                return FreeCellsCount != 0;
            }
        }
        public int TotalFreeCellsCount
        {
            get
            {
                // spec: Places libres ( Total avail. cells) : Places totales - Places utilisées  100% (sans distinction équipées ou pas)  (= Nb total de places à équiper + Nb de places disp. de suite)

                return this.CellsNeedingEquipmentCount + this.AvailableCellsCount;

                // so equivalent to:
                //return CellsCount - UsedCellsCount;
            }
        }

        public int UsedCellsCount
        {
            get
            {
                return this.CellsCount - this.FreeCellsCount;
            }
        }

        public int EquippedCellsCount
        {
            get
            {
                return this.Rooms.Sum( r => r.Cells.Where( c => c.IsEquipped ).Count() );
            }
        }
        public int CellsNeedingEquipmentCount
        {
            get
            {
                return this.CellsCount - this.EquippedCellsCount;
            }
        }

        public List<Cell> DeskSharingCells
        {
            get
            {
                return this.Rooms.SelectMany( r => r.Cells.Where( c => c.CellTypeID == Constants.DeskSharingCellTypeId ) ).ToList();
            }
        }
        public int DeskSharingCellsCount
        {
            get
            {
                return this.DeskSharingCells.Count;
            }
        }
        public int PeopleInDeskSharingCellsCount
        {
            get
            {
                return this.DeskSharingCells.Sum( c => c.People.Count );
            }
        }

        public int VisitorCellsCount
        {
            get
            {
                return this.Rooms.Sum( r => r.Cells.Where( c => c.CellTypeID == Constants.VisitorCellTypeId ).Count() );
            }
        }

        public int AvailableCellsCount
        {
            get
            {
                return this.EquippedCellsCount - this.UsedCellsCount - VisitorCellsCount;
            }
        }

        public void RaiseFreeCellsChanges()
        {
            this.RaisePropertyChanged( "FreeCellsCount" );
            this.RaisePropertyChanged( "FreeCells" );
            this.RaisePropertyChanged( "HasFreeCells" );
            this.Site.RaiseAllChanges();
        }

        /// <summary>
        /// In what kind of change is this floor involved ?
        /// </summary>
        public HypothesisChangeType HypothesisChangeType {
            
            get {

                // if the associated floor is in a site deleted by the hypothesis, returns 'deleted'
                Site s = Site;
                if (s != null) {
                    SiteHypothesis sh = s.GetInvolvedSiteHypothesis();
                    if (sh != null && sh.HypothesisChangeType == HypothesisChangeType.Deleted) {
                        return HypothesisChangeType.Deleted;
                    }
                }

                FloorHypothesis floorHyp = GetInvolvedFloorHypothesis();
                if (floorHyp == null) {

                    return HypothesisChangeType.Unchanged;
                }

                return floorHyp.HypothesisChangeType;
            }
        }

        public static event EventHandler<DateChangeEventArgs> FloorDateChanged;

        /// <summary>
        /// If we have our own <c>RentStart</c>, return 'Normal'; if it comes from our site, return Italic
        /// </summary>
        public FontStyle FontStyleRentStart {
            get {

                if (RentStart != null) {
                    
                    return FontStyles.Normal;
                }

                FloorHypothesis fh = GetInvolvedFloorHypothesis();
                if (fh != null && fh.HypothesisChangeType == HypothesisChangeType.Modified) {
                    return FontStyles.Normal;
                }

                if (Site != null && Site.DisplayedRentStart != null) {
                    return FontStyles.Italic;
                }

                return FontStyles.Normal;
            }
        }

        /// <summary>
        /// Same as <c>RentStart</c>, except if we are involved in a change hypothesis which has a RentStart, in which returns it returns <c>FloorHypothesis.RentStart</c>.
        /// If we don't have a RentStart but our Site has one, returns it.
        /// </summary>
        public DateTime? DisplayedRentStart {
            get {

                FloorHypothesis fh = GetInvolvedFloorHypothesis();
                if (fh != null && fh.HypothesisChangeType == HypothesisChangeType.Modified) {
                    if (fh.RentStart != null) {
                        return fh.RentStart;
                    }
                }

                return RentStart ?? (Site == null ? null : Site.DisplayedRentStart);
            }

            set {

                if (! StateHelper.IsReady) {
                    return;
                }

                FloorHypothesis fh = GetInvolvedFloorHypothesis();
                if (fh == null) {

                    FireFloorDateChanged(value, null);
                } else if (fh.HypothesisChangeType == HypothesisChangeType.Modified) {

                    fh.RentStart = value;
                } else {

                    RentStart = value;
                }

                InformRentStartChange();
            }
        }

        /// <summary>
        /// Same as <c>RentEnd</c>, except if we are involved in a change hypothesis which has a RentEnd, in which returns it returns <c>FloorHypothesis.RentEnd</c>.
        /// If we don't have a RentEnd but our Site has one, returns it.
        /// </summary>
        public DateTime? DisplayedRentEnd {
            get {

                FloorHypothesis fh = GetInvolvedFloorHypothesis();
                if (fh != null && fh.HypothesisChangeType == HypothesisChangeType.Modified) {

                    if (fh.RentEnd != null) {
                        return fh.RentEnd;
                    }
                }

                return RentEnd ?? (Site == null ? null : Site.DisplayedRentEnd);
            }

            set {

                if (!StateHelper.IsReady) {
                    return;
                }

                FloorHypothesis fh = GetInvolvedFloorHypothesis();
                if (fh == null) {

                    FireFloorDateChanged(null, value);
                } else if (fh.HypothesisChangeType == HypothesisChangeType.Modified) {

                    fh.RentEnd = value;
                } else {

                    RentEnd = value;
                }

                InformRentEndChange();
            }
        }

        public bool IsRentInfoEditable {
            get {
                return StateHelper.CurrentHypothesis != null;
            }
        }

        private void FireFloorDateChanged(DateTime? startDate, DateTime? endDate) {
            if (FloorDateChanged != null) {
                
                FloorDateChanged(this, new DateChangeEventArgs(startDate, endDate));
            }
        }

        public FloorHypothesis GetInvolvedFloorHypothesis() {

            if (FloorHypotheses.Count == 0) {

                return null;
            }

            if (StateHelper.CurrentHypothesis == null) {
                return null;
            }

            return StateHelper.CurrentHypothesis.FloorHypotheses.FirstOrDefault(fh => fh.Floor == this);
        }

        public void RaiseHypothesisChange() {

            RaisePropertyChanged("HypothesisChangeType");
            InformRentStartChange();
            InformRentEndChange();
        }

        public void InformRentStartChange() {
            RaisePropertyChanged("DisplayedRentStart");
            RaisePropertyChanged("FontStyleRentStart");
        }

        public void InformRentEndChange() {
            RaisePropertyChanged("DisplayedRentEnd");
            RaisePropertyChanged("FontStyleRentEnd");
        }
    }

    [DebuggerDisplay("Name = {Name}")]
    public partial class Site
    {
        public static bool IsSite { get { return true; } } // HACK used to display "total" in ReportView. Much simpler than having site implement ICloneable.

        // has to match the enum UpdateRule
        public static readonly List<string> RpisRules = new List<string>(2) { "By phone", "By visa", "ignored"};

        public int CellsCount
        {
            get
            {
                return Floors.Sum( f => f.CellsCount );
            }
        }

        public int FreeCellsCount
        {
            get
            {
                return Floors.Sum( f => f.FreeCellsCount );
            }
        }
        public string FreeCells
        {
            get
            {
                return "(" + FreeCellsCount + " free)";
            }
        }
        public bool HasFreeCells
        {
            get
            {
                return FreeCellsCount != 0;
            }
        }
        public int TotalFreeCellsCount
        {
            get
            {
                return Floors.Sum( f => f.TotalFreeCellsCount );
            }
        }

        public int UsedCellsCount
        {
            get
            {
                return Floors.Sum( f => f.UsedCellsCount );
            }
        }

        public int EquippedCellsCount
        {
            get
            {
                return Floors.Sum( f => f.EquippedCellsCount );
            }
        }
        public int CellsNeedingEquipmentCount
        {
            get
            {
                return Floors.Sum( f => f.CellsNeedingEquipmentCount );
            }
        }

        public List<Cell> DeskSharingCells
        {
            get
            {
                List<Cell> deskSharingCells = new List<Cell>();
                foreach ( Floor f in this.Floors )
                {
                    deskSharingCells.AddRange( f.DeskSharingCells );
                }
                return deskSharingCells;
            }
        }
        public int DeskSharingCellsCount
        {
            get
            {
                return this.DeskSharingCells.Count;
            }
        }
        public int PeopleInDeskSharingCellsCount
        {
            get
            {
                return Floors.Sum( f => f.PeopleInDeskSharingCellsCount );
            }
        }

        public int VisitorCellsCount
        {
            get
            {
                return Floors.Sum( f => f.VisitorCellsCount );
            }
        }

        public int AvailableCellsCount
        {
            get
            {
                return Floors.Sum( f => f.AvailableCellsCount );
            }
        }

        public string DisplayedPhoneRanges {
            get {
                if (PhoneRanges == null || PhoneRanges.Count == 0) {
                    return String.Empty;
                }

                IEnumerable<string> ranges = PhoneRanges.Select(pr => pr.Prefix.ToString());
                return String.Join(", ", ranges);
            }
            set {

                if (! StateHelper.IsReady) {
                    return; // ignore while not ready
                }

                string[] ranges = value.Split(',', ';');

                // init algorithm
                foreach (PhoneRange pr in PhoneRanges) {
                    pr.IsSelectedByUser = false;
                }

                // can we find these ranges ?
                foreach (string range in ranges) {

                    bool found = false;
                    int hundred;
                    if (Int32.TryParse(range, out hundred)) {

                        // loop through the existing ranges
                        foreach (PhoneRange pr in PhoneRanges) {
                            if (pr.Prefix == hundred) {
                                found = true;
                                pr.IsSelectedByUser = true;
                                break;
                            }
                        }
                    } else {
                        continue; // analyze next one
                    }

                    if (! found) {
                        PhoneRange newPR = new PhoneRange {Prefix = hundred, IsSelectedByUser = true, Site=this};
                        PhoneRanges.Add(newPR);
                    }
                }

                // remove unwanted elements
                int index = 0;
                while (index < PhoneRanges.Count) {

                    PhoneRange pr = PhoneRanges.ElementAt(index);
                    if (!pr.IsSelectedByUser) {
                        StateHelper.Context.PhoneRanges.Remove(pr);
                    } else {
                        index++; // go to next PhoneRange
                    }
                }
            }
        }

        /// <summary>
        /// In what kind of change is this site involved ?
        /// </summary>
        public HypothesisChangeType HypothesisChangeType {
            
            get {

                SiteHypothesis siteHyp = GetInvolvedSiteHypothesis();
                if (siteHyp == null) {

                    return HypothesisChangeType.Unchanged;
                }

                return siteHyp.HypothesisChangeType;
            }
        }

        public static event EventHandler<DateChangeEventArgs> SiteDateChanged;

        /// <summary>
        /// Same as <c>RentStart</c>, except if we are involved in a change hypothesis that has a RentStart, 
        /// in which case it returns <c>SiteHypothesis.RentStart</c>
        /// </summary>
        public DateTime? DisplayedRentStart {
            get {

                SiteHypothesis sh = GetInvolvedSiteHypothesis();
                if (sh != null && sh.HypothesisChangeType == HypothesisChangeType.Modified) {

                    if (sh.RentStart != null) {
                        return sh.RentStart;
                    }
                }

                return RentStart;
            }

            set {

                if (! StateHelper.IsReady) {
                    return;
                }

                SiteHypothesis sh = GetInvolvedSiteHypothesis();
                if (sh == null) {

                    FireSiteDateChanged(value, null);
                } else if (sh.HypothesisChangeType == HypothesisChangeType.Modified) {

                    sh.RentStart = value;
                } else {

                    RentStart = value;
                }

                InformRentStartChange();
            }
        }

        public void InformRentStartChange() {
            RaisePropertyChanged("DisplayedRentStart");
            foreach (Floor f in Floors) {
                f.InformRentStartChange();
            }
        }

        /// <summary>
        /// Same as <c>RentEnd</c>, except if we are involved in a change hypothesis that has a RentEnd, 
        /// in which case it returns <c>SiteHypothesis.RentEnd</c>
        /// </summary>
        public DateTime? DisplayedRentEnd {
            get {

                SiteHypothesis sh = GetInvolvedSiteHypothesis();
                if (sh != null && sh.HypothesisChangeType == HypothesisChangeType.Modified) {
                    if (sh.RentEnd != null) {
                        return sh.RentEnd;
                    }
                }

                return RentEnd;
            }

            set {
                
                if (! StateHelper.IsReady) {
                    return;
                }

                SiteHypothesis sh = GetInvolvedSiteHypothesis();
                if (sh == null) {

                    FireSiteDateChanged(null, value);
                } else if (sh.HypothesisChangeType == HypothesisChangeType.Modified) {

                    sh.RentEnd = value;
                } else {

                    RentEnd = value;
                }

                InformRentEndChange();
            }
        }

        public void InformRentEndChange() {
            RaisePropertyChanged("DisplayedRentEnd");
            foreach (Floor f in Floors) {
                f.InformRentEndChange();
            }
        }

        public bool IsRentInfoEditable {
            get {
                return StateHelper.CurrentHypothesis != null;
            }
        }

        private void FireSiteDateChanged(DateTime? startDate, DateTime? endDate) {
            if (SiteDateChanged != null) {
                
                SiteDateChanged(this, new DateChangeEventArgs(startDate, endDate));
            }
        }

        public SiteHypothesis GetInvolvedSiteHypothesis() {

            if (SiteHypotheses.Count == 0) {

                return null;
            }

            if (StateHelper.CurrentHypothesis == null) {
                return null;
            }

            return StateHelper.CurrentHypothesis.SiteHypotheses.FirstOrDefault(sh => sh.Site == this);
        }

        public void RaiseAllChanges()
        {
            RaisePropertyChanged("CellsCount" );
            RaisePropertyChanged("FreeCells" );
            RaisePropertyChanged("FreeCellsCount" );
            RaisePropertyChanged("HasFreeCells" );
            RaisePropertyChanged("TotalFreeCellsCount" );
            RaisePropertyChanged("UsedCellsCount" );
            RaisePropertyChanged("EquippedCellsCount" );
            RaisePropertyChanged("CellsNeedingEquipmentCount" );
            RaisePropertyChanged("DeskSharingCellsCount" );
            RaisePropertyChanged("PeopleInDeskSharingCellsCount" );
            RaisePropertyChanged("VisitorCellsCount" );
            RaisePropertyChanged("AvailableCellsCount" );
            RaisePropertyChanged("IsEnabled");
            RaisePropertyChanged("DisplayedPhoneRanges");
            RaisePropertyChanged("HypothesisChangeType");
        }

        public void RaiseHypothesisChange() {
            
            RaisePropertyChanged("HypothesisChangeType");
        }
    }

    public partial class SiteHypothesis {
        
        public HypothesisChangeType HypothesisChangeType {
            
            get {
                return (HypothesisChangeType) ChangeType;
            }
            set {
                ChangeType = (short) value;
            }
        }
    }

    public partial class FloorHypothesis {
        
        public HypothesisChangeType HypothesisChangeType {
            
            get {
                return (HypothesisChangeType) ChangeType;
            }
            set {
                ChangeType = (short) value;
            }
        }
    }

    public partial class Hypothesis {

        // The constructor is *not* called when an object is deserialized. 
        // And it's not written  anywhere in the method descriptions...
        [OnDeserialized]
        new public void OnDeserialized(StreamingContext value) {
            OnCreated();
        }

        partial void OnCreated() {
            INotifyCollectionChanged ncc = SiteHypotheses;
            // we are called very early in the process of updating the collection =>
            // postpone the handling in order to let all pointers be valid
            // (previously SiteHypothesis.Site was often null)
            ncc.CollectionChanged += delegate {
                SynchronizationContext.Current.Post(
                    state => {
                        RaisePropertyChanged("HypothesisContent");
                        RaisePropertyChanged("DetailedHypothesisContent");
                    }, null);
            };

            ncc = FloorHypotheses;
            ncc.CollectionChanged += delegate {
                SynchronizationContext.Current.Post(
                    state => {
                        RaisePropertyChanged("HypothesisContent");
                        RaisePropertyChanged("DetailedHypothesisContent");
                    }, null);
            };
        }

        public string HypothesisContent {
            get {
                StringBuilder sb = new StringBuilder();

                sb.AppendFormat("{0} site change{1}", SiteHypotheses.Count, GetPlural(SiteHypotheses.Count));
                sb.AppendFormat(" / {0} floor change{1}", FloorHypotheses.Count, GetPlural(FloorHypotheses.Count));

                return sb.ToString();
            }
        }

        public string DetailedHypothesisContent {
            get {

                string[] details = new string[SiteHypotheses.Count + FloorHypotheses.Count];
                int index = 0;

                foreach (SiteHypothesis sh in SiteHypotheses) {

                    string siteName = sh.Site == null ? String.Empty : sh.Site.Name;

                    switch (sh.HypothesisChangeType) {
                            
                        case HypothesisChangeType.Added:
                            details[index] = String.Format("- added site {0}", siteName);
                            break;

                        case HypothesisChangeType.Deleted:
                            details[index] = String.Format("- deleted site {0}", siteName);
                            break;

                        case HypothesisChangeType.Modified:
                            details[index] = String.Format("- changed rent date of site {0}", siteName);
                            break;
                    }

                    index++;
                }


                foreach (FloorHypothesis sh in FloorHypotheses) {

                    string floorNumber = sh.Floor == null ? String.Empty : sh.Floor.Number.ToString();

                    switch (sh.HypothesisChangeType) {
                            
                        case HypothesisChangeType.Added:
                            details[index] = String.Format("- added floor {0}", floorNumber);
                            break;

                        case HypothesisChangeType.Deleted:
                            details[index] = String.Format("- deleted floor {0}", floorNumber);
                            break;

                        case HypothesisChangeType.Modified:
                            details[index] = String.Format("- changed rent date of floor {0}", floorNumber);
                            break;
                    }

                    index++;
                }

                return String.Join(Environment.NewLine, details);
            }
        }

        private string GetPlural(int count) {

            return count > 1 ? "s" : String.Empty;
        }
    }

    [DebuggerDisplay("Prefix = {Prefix}")]
    public partial class PhoneRange {

        /// <summary>
        /// Used when the user enters new ranges
        /// </summary>
        public bool IsSelectedByUser { get; set; }
    }

    [DebuggerDisplay("{Visa}")]
    public partial class Person
    {
        public void ChangeCell( Cell newCell )
        {
            Cell oldCell = this.Cell;
            this.Cell = newCell;
            if ( oldCell != null ) oldCell.RaiseAllChanges();

            newCell.RaiseAllChanges();
            this.NewWorkPlace = this.Cell.Room.Floor.Site.Name + " " + this.Cell.FullNumber;
            if ( oldCell != null && oldCell.Room.Floor.Site != newCell.Room.Floor.Site && this.PhoneNumber == ( (Person) this.GetOriginal() ).PhoneNumber )
            {
                // If someone moves to another building, his phone number needs to be changed
                this.PhoneNumber += '!';
            }
        }

        public override bool Equals( object obj )
        {
            Person p = (Person) obj;
            return this.ID == p.ID;
        }
        public override int GetHashCode()
        {
            return this.ID.GetHashCode();
        }

        public string OldPhoneNumber
        {
            get
            {
                Person p = (Person) this.GetOriginal();
                return p != null ? p.PhoneNumber : null;
            }
        }
        public bool HasPhoneNumberChanged
        {
            get
            {
                return ( this.PhoneNumber != this.OldPhoneNumber );
            }
        }

        public void RaisePhoneAndWorkPlaceChanges()
        {
            RaisePropertyChanged( "NewWorkPlace" );
            RaisePropertyChanged( "PhoneNumber" );
            RaisePropertyChanged( "HasPhoneNumberChanged" );
        }

        public bool IsEnabled {
            get {
                return (Flags & Constants.AdsDisabledAccount) == 0;
            }
            set {
                if (value)
                {
                    Flags &= ~Constants.AdsDisabledAccount;
                }
                else {
                    Flags |= Constants.AdsDisabledAccount;
                }
                RaisePropertyChanged("IsEnabled");
            }
        }
    }

    public static class StateHelper {
        public static bool IsReady { get; set; }
        public static Hypothesis CurrentHypothesis { get; set; }
        public static RoomContext Context { get; set; }
    }

    // IRefreshable is for the Refresh button, ICanBeBusy (notîce the stupid pun) is for the BusyIndicator
    public interface IRefreshable
    {
        void RefreshData();
    }
    public interface ICanBeBusy
    {
        bool IsBusy { get; }
        event EventHandler IsBusyChanged;
    }

    public interface IChangeChildren {

        FrameworkElement Root { get;  }
        void Add(UIElement elt);
        void RemoveById(string name);
    }
    public interface IWantFocusNotification {
        
        void HasLostFocus();

        void HasGotFocus();
    }

    public enum SeatOrientation : byte {
        North, West, East, South,
    }

    [Flags]
    public enum PointType {

        WallStart = 1,
        WallJunction = 2,
        WallEnd = 4,
        DoorPivot = 8,
        DoorOpened = 16, // usually inside the room
        DoorClosed = 32,
        MovableWallStart = 64,
        MovableWallJunction = 128,
        MovableWallEnd = 256,
    }

    /// <summary>
    /// Information about <c>Site.FloorDateChanged</c> event
    /// </summary>
    public class DateChangeEventArgs : EventArgs {

        public DateTime? StartDate { get; private set; }
        public DateTime? EndDate { get; private set; }

        public DateChangeEventArgs(DateTime? startDate, DateTime? endDate) {

            StartDate = startDate;
            EndDate = endDate;
        }
    }
}

namespace RoomManagement.Web.Services {

    public partial class RoomContext {

        /// <summary>
        /// Sets a 5-minute timeout (instead of the default 1 minute)
        /// </summary>
        partial void OnCreated() {

            if (! DesignerProperties.GetIsInDesignMode(Application.Current.RootVisual))
                ((WebDomainClient<IRoomServiceContract>)this.DomainClient).ChannelFactory.Endpoint.Binding.SendTimeout = new TimeSpan(0, 5, 0);
            
        }
        
    }

}