﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Net;
using System.ServiceModel.DomainServices.Client;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Elca.MvvmHelpers;
using RoomManagement.Views;
using RoomManagement.Web.Models;
using RoomManagement.Web.Services;

namespace RoomManagement.ViewModels {
    public class HypothesisRoomsViewModel : BaseViewModel, IRefreshable, ICanBeBusy, IWantFocusNotification {
     
        private readonly RoomContext _rc = new RoomContext();

        private EntitySet<Hypothesis> _hypotheses;
        public EntitySet<Hypothesis> Hypotheses {
            get { return _hypotheses; }
            set {
                _hypotheses = value;
                RaisePropertyChanged("Hypotheses");
            }
        }

        private Hypothesis _currentHypothesis;
        public Hypothesis CurrentHypothesis {
            get {
                return _currentHypothesis;
            }
            set {
                _currentHypothesis = value;
                StateHelper.CurrentHypothesis = value;
                RaisePropertyChanged("CurrentHypothesis");
                RaisePropertyChanged("IsSiteEditionEnabled");
                RecomputeSites();
            }
        }

        private ObservableCollection<Site> _sites;
        public ObservableCollection<Site> Sites {
            get { return _sites; }
            set {
                _sites = value;
                RaisePropertyChanged("Sites");
            }
        }

        private Site _currentSite;
        public Site CurrentSite {
            get {
                return _currentSite;
            }
            set {
                if (value != _currentSite) {
                    _currentSite = value;
                    RaisePropertyChanged("CurrentSite");
                    RaisePropertyChanged("IsSiteAdditionEnabled");
                    RaisePropertyChanged("IsSiteDeletionEnabled");
                    RaisePropertyChanged("DeleteSiteCommandText");
                    RaisePropertyChanged("IsFloorAdditionEnabled");
                    RaisePropertyChanged("IsFloorDeletionEnabled");
                    RecomputeFloors();
                }
            }
        }

        private EntitySet<RoomType> _roomTypes;
        public EntitySet<RoomType> RoomTypes {
            get { return _roomTypes; }
            set {
                _roomTypes = value;
                RaisePropertyChanged("RoomTypes");
            }
        }
        private EntitySet<CellType> _cellTypes;
        public EntitySet<CellType> CellTypes {
            get { return _cellTypes; }
            set {
                _cellTypes = value;
                RaisePropertyChanged("CellTypes");
            }
        }

        private ObservableCollection<Floor> _floors;
        public ObservableCollection<Floor> Floors {
            get { return _floors; }
            set {
                _floors = value;
                RaisePropertyChanged("Floors");
            }
        }

        private Floor _currentFloor;
        public Floor CurrentFloor {
            get {
                return _currentFloor;
            }
            set {
                if (value != _currentFloor) {
                    _currentFloor = value;
                    RaisePropertyChanged("CurrentFloor");
                    RaisePropertyChanged("DeleteFloorCommandText");
                    RaisePropertyChanged("IsFloorAdditionEnabled");
                    RaisePropertyChanged("IsFloorDeletionEnabled");
                }
            }
        }

        /// <summary>
        /// The text to display in the delete site button 
        /// </summary>
        public string DeleteSiteCommandText {
            get {

                return GetDeleteCommandTextFromChangeType(CurrentSite == null
                                                           ? HypothesisChangeType.Unchanged
                                                           : CurrentSite.HypothesisChangeType);
            }
        }

        /// <summary>
        /// The text to display in the delete site button 
        /// </summary>
        public string DeleteFloorCommandText {

            get {

                HypothesisChangeType hct;

                if (!IsFloorDeletionEnabled) {
                    hct = HypothesisChangeType.Unchanged;
                } else {
                    hct = CurrentFloor == null
                              ? HypothesisChangeType.Unchanged
                              : CurrentFloor.HypothesisChangeType;
                }

                return GetDeleteCommandTextFromChangeType(hct);
            }
        }

        private string GetDeleteCommandTextFromChangeType(HypothesisChangeType? hct) {
            
            string result;

            switch (hct) {

                case HypothesisChangeType.Added:
                    result = "Cancel add";
                    break;

                case HypothesisChangeType.Deleted:
                    result = "Cancel delete";
                    break;

                case HypothesisChangeType.Modified:
                    result = "Cancel change";
                    break;

                default:
                    // default value
                    result = "Delete";
                    break;
            }
            
            return result;
        }

        public ICommand AddHypothesisCommand { get; private set; }

        public ICommand DeleteHypothesisCommand { get; private set; }
        public ICommand AddSiteCommand { get; private set; }
        public ICommand DeleteSiteCommand { get; private set; }
        public ICommand AddFloorCommand { get; private set; }
        public ICommand DeleteFloorCommand { get; private set; }
        public ICommand RejectCommand { get; private set; }
        public ICommand SaveCommand { get; private set; }

        private bool IsSiteAdditionEnabled {
            get {
                return CurrentHypothesis != null;
            }
        }

        private bool IsSiteDeletionEnabled {
            get {
                return IsSiteAdditionEnabled && CurrentSite != null;
            }
        }

        private bool IsFloorAdditionEnabled {
            get {
                bool basicCondition = CurrentHypothesis != null && CurrentSite != null;
                if (basicCondition) {
                    // check that the site is not deleted by the current hypothesis
                    SiteHypothesis sh = CurrentSite.GetInvolvedSiteHypothesis();
                    if (sh != null) {

                        // mmmh.. this is no longer a basic condition
                        basicCondition = sh.HypothesisChangeType != HypothesisChangeType.Deleted;
                    }
                }
                return basicCondition;
            }
        }

        private bool IsFloorDeletionEnabled {
            get {

                // almot identical to IsFloorAdditionEnabled, except that there must be a current floor
                return IsFloorAdditionEnabled && CurrentFloor != null;
            }
        }

        public HypothesisRoomsViewModel() {

            StateHelper.IsReady = false;
            StateHelper.Context = _rc;
            Site.SiteDateChanged += OnSiteDateChanged;
            Floor.FloorDateChanged += OnFloorDateChanged;

            _hypotheses = _rc.Hypothesis;

            _rc.PropertyChanged += OnContextPropertyChanged;

            // Question: should I give as dependent properties the property name that is called here, or the properties used to implement the IsEnabledXXX
            AddHypothesisCommand = new ConditionalCommand(CreateHypothesis, () => IsSiteAdditionEnabled, this, "IsSiteAdditionEnabled");
            DeleteHypothesisCommand = new ConditionalCommand(DeleteCurrentHypothesis, () => IsSiteDeletionEnabled, this, "IsSiteDeletionEnabled");

            AddSiteCommand = new ConditionalCommand(AddSite, () => IsSiteAdditionEnabled, this, "IsSiteAdditionEnabled");
            DeleteSiteCommand = new ConditionalCommand(DeleteSite, () => IsSiteDeletionEnabled, this, "IsSiteDeletionEnabled");
            AddFloorCommand = new ConditionalCommand(AddFloor, () => IsFloorAdditionEnabled, this, "IsFloorAdditionEnabled");
            DeleteFloorCommand = new ConditionalCommand(DeleteFloor, () => IsFloorDeletionEnabled, this, "IsFloorDeletionEnabled");
            RejectCommand = new ConditionalCommand(Reject, () => HasChanges, this, "HasChanges");
            SaveCommand = new ConditionalCommand(Save, () => HasChanges, this, "HasChanges");

            RefreshData();
        }

        private bool HasChanges { get { return (_rc.HasChanges); } }
        public bool IsBusy { get { return _rc.IsLoading || _rc.IsSubmitting; } }

        public event EventHandler IsBusyChanged;

        private void OnContextPropertyChanged(object sender, PropertyChangedEventArgs e) {

            if (e.PropertyName == "IsLoading") {
                
                FireBusyChanged();

                StateHelper.IsReady = !_rc.IsLoading;
            } else if (e.PropertyName == "IsSubmitting") {

                FireBusyChanged();
            } else if (e.PropertyName == "HasChanges") {

                RaisePropertyChanged("HasChanges");
            }
        }

        private void FireBusyChanged() {
            if (IsBusyChanged != null) {
                IsBusyChanged(this, EventArgs.Empty);
            }
        }

        private void OnApplyHypothesis(object sender, RoutedEventArgs e) {
            ApplyHypothesis(CurrentHypothesis);
        }

        private void DeleteCurrentHypothesis() {
            DeleteHypothesis(CurrentHypothesis);
        }

        private void DeleteHypothesis(Hypothesis h) {

            if (h == null) {
                return;
            }

            SilverMessageBox smb = SilverMessageBox.Show("Are you REALLY sure that you want to do delete this hypothesis ? All related modifications will be deleted.", "Yes", button2Text: "No");
            smb.Closed += delegate {
                if (smb.DialogResult == true) // Yes
                {
                    while (CurrentHypothesis.FloorHypotheses.Count > 0) {
                        FloorHypothesis sh = CurrentHypothesis.FloorHypotheses.ElementAt(0);
                        CancelFloorHypothesis(sh);
                    }
                    while (CurrentHypothesis.SiteHypotheses.Count > 0) {
                        SiteHypothesis sh = CurrentHypothesis.SiteHypotheses.ElementAt(0);
                        CancelSiteHypothesis(sh);
                    }
                    _rc.Hypothesis.Remove(CurrentHypothesis);
                }
            };
        }

        private void ApplyHypothesis(Hypothesis h) {

            if (h == null) {
                return;
            }

            foreach (SiteHypothesis sh in h.SiteHypotheses) {
                
                switch (sh.HypothesisChangeType) {
                    case HypothesisChangeType.Added:

                        // nothing special to do. The data is already there
                        break;

                    case HypothesisChangeType.Modified:

                        // copy rent data from SiteHypothesis to Site
                        Site sMod = sh.Site;
                        sMod.RentStart = sh.RentStart;
                        sMod.RentEnd = sh.RentEnd;
                        break;

                    case HypothesisChangeType.Deleted:

                        // as a new site had been added for this case, just delete it
                        Site sDel = sh.Site;
                        _rc.Sites.Remove(sDel);
                        break;

                    default:
                        break;
                }

                _rc.SiteHypothesis.Remove(sh);
            }

            _rc.Hypothesis.Remove(h);

            RecomputeSites();
        }

        private void CreateHypothesis() {

            Hypothesis hyp = new Hypothesis {CreationDate = DateTime.Now};
            _rc.Hypothesis.Add(hyp);
            CurrentHypothesis = hyp;
        }

        private void RecomputeSites() {

            Hypothesis hyp = CurrentHypothesis;

            ObservableCollection<Site> newSites;

            if (hyp == null) {

                newSites = new ObservableCollection<Site>(_rc.Sites.Where(s => s.SiteHypotheses.Count == 0));
            } else {

                // take all sites, except those that have been added by other hypotheses
                newSites = new ObservableCollection<Site>();

                foreach (Site s in _rc.Sites) {

                    bool isToAdd = true;

                    if (s.SiteHypotheses.Count > 0) {

                        // site is involved in at least one hypothesis
                        // do not add if all of them are additions
                        isToAdd = false;

                        foreach (SiteHypothesis sh in s.SiteHypotheses) {

                            if (sh.HypothesisChangeType != HypothesisChangeType.Added ||
                                sh.Hypothesis == hyp) {

                                isToAdd = true;
                                break;
                            }
                        }
                    }

                    if (isToAdd) {

                        newSites.Add(s);
                    }
                }
            }

            Sites = new ObservableCollection<Site>(newSites.OrderBy(s => s.Name));
        }

        private void RecomputeFloors() {

            Hypothesis hyp = CurrentHypothesis;

            ObservableCollection<Floor> newFloors;

            if (hyp == null) {

                newFloors = new ObservableCollection<Floor>(_rc.Floors.Where(f => f.Site == CurrentSite && f.FloorHypotheses.Count == 0));
            } else {


                // floors related to the current hypothesis (and the current site)
                newFloors = new ObservableCollection<Floor>(
                    _rc.Floors.Where(
                        f => f.Site == CurrentSite && f.FloorHypotheses.Any(sh => sh.Hypothesis == hyp)));

                // add sites unrelated to a hypothesis if we don't have a related site already
                foreach (Floor f in _rc.Floors.Where(fl => fl.Site == CurrentSite && fl.FloorHypotheses.Count == 0)) {

                    Floor currentFloor = f;
                    if (!hyp.FloorHypotheses.Any(sh => sh.Floor == currentFloor)) {
                        newFloors.Add(currentFloor);
                    }
                }
            }

            Floors = new ObservableCollection<Floor>(newFloors.OrderBy(s => s.Number));
            if (Floors.Count > 0) {
                CurrentFloor = Floors[0];
            }
        }

        public void RefreshData() {

            StateHelper.IsReady = false;

            _rc.RefreshSettingsData();
            LoadHypotheses();
            LoadSites(_rc.GetAllSitesQuery());
            LoadCellTypes();
            LoadRoomTypes();
        }

        private void LoadHypotheses() {
            _rc.Load(_rc.GetHypothesesQuery(true), LoadBehavior.RefreshCurrent, OnLoadedHypotheses, null);
        }
        private void OnLoadedHypotheses(LoadOperation e) {
            if (e.HasError) {
                SilverMessageBox.Show(Constants.ErrorMessage, "OK", e.Error.ToString());
                e.MarkErrorAsHandled();
                return;
            }
            Hypotheses = _rc.Hypothesis;
            CurrentHypothesis = Hypotheses.FirstOrDefault();
            _rc.Load(_rc.GetPeopleQuery(), LoadBehavior.RefreshCurrent, OnLoadedPeople, null);
        }
        private void LoadSites(EntityQuery<Site> query) {
            _rc.Load(query, LoadBehavior.RefreshCurrent, OnLoadedSites, null);
        }
        private void LoadCellTypes() {
            _rc.Load(_rc.GetCellTypesQuery(), LoadBehavior.RefreshCurrent, OnLoadedTypes, null);
        }
        private void LoadRoomTypes() {
            _rc.Load(_rc.GetRoomTypesQuery(), LoadBehavior.RefreshCurrent, OnLoadedTypes, null);
        }
        private static void OnLoadedTypes(LoadOperation e) {
            if (e.HasError) {
                SilverMessageBox.Show(Constants.ErrorMessage, "OK", e.Error.ToString());
                e.MarkErrorAsHandled();
                return;
            }
        }
        private void OnLoadedSites(LoadOperation e) {
            if (e.HasError) {
                SilverMessageBox.Show(Constants.ErrorMessage, "OK", e.Error.ToString());
                e.MarkErrorAsHandled();
                return;
            }
            RecomputeSites();
            _rc.Load(_rc.GetPeopleQuery(), LoadBehavior.RefreshCurrent, OnLoadedPeople, null);
        }
        private void OnLoadedPeople(LoadOperation e) {
            if (e.HasError) {
                SilverMessageBox.Show(Constants.ErrorMessage, "OK", e.Error.ToString());
                e.MarkErrorAsHandled();
                return;
            }

            foreach (Site s in _rc.Sites) {
                s.RaiseAllChanges();
            }
        }

        private void Save() {
            // SubmitChanges will merge data from the server to the client (here). 
            // As Site.DisplayedPhoneRanges is unknown at the server side, it will be set to null by RIA Services =>
            // Tell Site to ignore the future call to Site.DisplayedPhoneRanges = null
            StateHelper.IsReady = false;
            _rc.SubmitChanges(delegate(SubmitOperation so) {
                StateHelper.IsReady = true;
                if (so.HasError) {
                    StringBuilder sb = new StringBuilder("Error while saving changes to room data:" + Environment.NewLine);
                    foreach (Entity en in so.EntitiesInError) {
                        if (en.HasValidationErrors) {
                            foreach (ValidationResult vri in en.ValidationErrors) {
                                sb.AppendLine(vri.ErrorMessage);
                            }
                        }
                    }
                    SilverMessageBox.Show(sb.ToString());
                    so.MarkErrorAsHandled();
                }
            }, null);

        }

        private void Reject() {

            SilverMessageBox smb = SilverMessageBox.Show("Do you really want to reject all the changes you just made ?", "Yes", button2Text: "No");
            smb.Closed += delegate {
                if (smb.DialogResult == true) // Yes
                {
                    _rc.RejectChanges();
                } else return;
            };
        }

        public void HasLostFocus() {
            StateHelper.IsReady = false;
            Site.SiteDateChanged -= OnSiteDateChanged;
            Floor.FloorDateChanged -= OnFloorDateChanged;
        }

        public void HasGotFocus() {
            // not really interested
        }

        private void AddSite() {

            if (CurrentHypothesis == null) {
                return;
            }

            Site s = AddSiteToHypothesis(CurrentHypothesis);
        }

        private Site AddSiteToHypothesis(Hypothesis hyp) {
            Site s = new Site {
                                  Name = "Name",
                                  Location = "Fill me in !",
                              };
            _rc.Sites.Add(s);

            SiteHypothesis hc = new SiteHypothesis {
                HypothesisChangeType = HypothesisChangeType.Added,
                Hypothesis = hyp,
                Site = s,
            };
            _rc.SiteHypothesis.Add(hc);

            Sites.Add(s);
            return s;
        }

        private void DeleteSite() {

            if (CurrentHypothesis == null) {
                return;
            }

            if (CurrentSite == null) {
                return;
            }

            DeleteSiteFromHypothesis(CurrentSite, CurrentHypothesis);

            RecomputeSites();
        }

        private void DeleteSiteFromHypothesis(Site s, Hypothesis hyp) {

            SiteHypothesis sh = s.GetInvolvedSiteHypothesis();

            if (sh == null) {

                // "real" delete

                SiteHypothesis hc = new SiteHypothesis {
                    HypothesisChangeType = HypothesisChangeType.Deleted,
                    Hypothesis = hyp,
                    Site = s,
                };
                _rc.SiteHypothesis.Add(hc);
            } else {

                CancelSiteHypothesis(sh);
            }
        }

        private void CancelSiteHypothesis(SiteHypothesis sh) {
            _rc.SiteHypothesis.Remove(sh);
            if (sh.HypothesisChangeType == HypothesisChangeType.Added) {
                _rc.Sites.Remove(CurrentSite);
            }
        }

        private void OnSiteDateChanged(object sender, DateChangeEventArgs e) {

            Site s = (Site) sender;
            CreateSiteChangeHypothesis(s, e.StartDate, e.EndDate, CurrentHypothesis);
        }

        private void CreateSiteChangeHypothesis(Site s, DateTime? startDate, DateTime? endDate, Hypothesis hyp) {
            
            SiteHypothesis hc = new SiteHypothesis {
                HypothesisChangeType = HypothesisChangeType.Modified,
                Hypothesis = hyp,
                Site = s,
                RentStart = startDate,
                RentEnd = endDate,
            };

            _rc.SiteHypothesis.Add(hc);
            s.RaiseHypothesisChange();
            RaisePropertyChanged("DeleteSiteCommandText");
        }

        private void AddFloor() {

            if (CurrentHypothesis == null) {
                return;
            }
            if (CurrentSite == null) {
                return;
            }

            Floor f = AddFloorToHypothesis(CurrentSite, CurrentHypothesis);
        }

        private void DeleteFloor() {

            if (CurrentHypothesis == null) {
                return;
            }

            if (CurrentSite == null) {
                return;
            }

            if (CurrentFloor == null) {
                return;
            }

            DeleteFloorFromHypothesis(CurrentFloor, CurrentHypothesis);

            RecomputeFloors();
        }

        private void OnFloorDateChanged(object sender, DateChangeEventArgs e) {

            Floor f = (Floor) sender;
            CreateFloorChangeHypothesis(f, e.StartDate, e.EndDate, CurrentHypothesis);
        }

        private void DeleteFloorFromHypothesis(Floor f, Hypothesis hyp) {

            FloorHypothesis fh = f.GetInvolvedFloorHypothesis();

            if (fh == null) {

                // "real" delete

                FloorHypothesis hc = new FloorHypothesis {
                    HypothesisChangeType = HypothesisChangeType.Deleted,
                    Hypothesis = hyp,
                    Floor = f,
                };
                _rc.FloorHypothesis.Add(hc);
            } else {

                CancelFloorHypothesis(fh);
            }
        }

        private void CancelFloorHypothesis(FloorHypothesis fh) {
            _rc.FloorHypothesis.Remove(fh);
            if (fh.HypothesisChangeType == HypothesisChangeType.Added) {
                _rc.Floors.Remove(CurrentFloor);
            }
        }


        private void CreateFloorChangeHypothesis(Floor f, DateTime? startDate, DateTime? endDate, Hypothesis hyp) {
            
            FloorHypothesis hc = new FloorHypothesis {
                HypothesisChangeType = HypothesisChangeType.Modified,
                Hypothesis = hyp,
                Floor = f,
                RentStart = startDate,
                RentEnd = endDate,
            };

            _rc.FloorHypothesis.Add(hc);
            f.RaiseHypothesisChange();
            RaisePropertyChanged("DeleteFloorCommandText");
        }

        private Floor AddFloorToHypothesis(Site s, Hypothesis hyp) {
            Floor f = new Floor {
                Number = 999,
                Location = "Fill me in !",
            };
            s.Floors.Add(f);

            FloorHypothesis hc = new FloorHypothesis {
                HypothesisChangeType = HypothesisChangeType.Added,
                Hypothesis = hyp,
                Floor = f,
            };
            _rc.FloorHypothesis.Add(hc);

            Floors.Add(f);
            return f;
        }

    }
}
