﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Windows.Input;
using System.ComponentModel;

namespace Elca.MvvmHelpers {

    /// <summary>
    /// Simple command that is always executable
    /// </summary>
    public class SimpleCommand : ICommand {

        private readonly Action<object> _action;

        /// <summary>
        /// Constructor taking a delegate with parameter
        /// </summary>
        /// <param name="action">The action to perform when the command is executed</param>
        public SimpleCommand(Action<object> action) {

            _action = action;
        }

        /// <summary>
        /// Constructor taking a delegate without any parameter
        /// </summary>
        /// <param name="action">The action to perform when the command is executed</param>
        public SimpleCommand(Action action) {

            _action = delegate { action(); };
        }

        /// <summary>
        /// Implementation of <c>ICommand.Execute</c>
        /// </summary>
        /// <param name="parameter"></param>
        public void Execute(object parameter) {

            _action(parameter);
        }

        /// <summary>
        /// Implemenation of <c>ICommand.CanExecute</c>
        /// </summary>
        /// <param name="parameter">parameter, typically from XAML</param>
        /// <returns>True if the command is enabled</returns>
        public virtual bool CanExecute(object parameter) {
            return true;
        }

        /// <summary>
        /// Implemenation of <c>ICommand.CanExecute</c>
        /// </summary>
        public event EventHandler CanExecuteChanged;

        protected virtual void FireExecuteChanged() {

            if (CanExecuteChanged != null) {
                CanExecuteChanged(this, EventArgs.Empty);
            }
        }
    }

    /// <summary>
    /// A command that determines if it is enabled through a delegate
    /// </summary>
    public class ConditionalCommand : SimpleCommand {

        private Func<bool> _validation;
        private IList<string> _dependentProperties;

        /// <summary>
        /// Constructor taking 2 delegates, one with an action parameter
        /// </summary>
        /// <param name="action">The action to execute</param>
        /// <param name="validation">Determines if the command is enabled</param>
        /// <param name="notifier">The object that will send a notification</param>
        /// <param name="dependentProperties">The list of properties that can cause a change in the <c>CanExecute</c> value</param>
        public ConditionalCommand(Action<object> action, Func<bool> validation,
                                  INotifyPropertyChanged notifier,
                                  params string[] dependentProperties) : base(action) {
            SetupCommand(validation, notifier, dependentProperties);
        }

        private void SetupCommand(Func<bool> validation, INotifyPropertyChanged notifier, string[] dependentProperties) {

            Contract.Requires(validation != null);

            _validation = validation;

            if (notifier != null) {

                notifier.PropertyChanged += OnDependentObjectChanged;
                _dependentProperties = dependentProperties;
            }
        }

        /// <summary>
        /// Constructor taking 2 delegates without parameter
        /// </summary>
        /// <param name="action">The action to execute</param>
        /// <param name="validation">Determines if the command is enabled</param>
        /// <param name="notifier">The object that will send a notification</param>
        /// <param name="dependentProperties">The list of properties that can cause a change in the <c>CanExecute</c> value</param>
        public ConditionalCommand(Action action, Func<bool> validation,
                                  INotifyPropertyChanged notifier,
                                  params string[] dependentProperties) : base(action) {

            SetupCommand(validation, notifier, dependentProperties);
        }

        /// <summary>
        /// Is the command enabled ?
        /// </summary>
        /// <param name="parameter">Ignored parameter</param>
        /// <returns>True if the command is enabled</returns>
        public override bool CanExecute(object parameter) {
            return _validation();
        }

        private void OnDependentObjectChanged(object sender, PropertyChangedEventArgs pcea) {

            if (_dependentProperties.Contains(pcea.PropertyName)) {
                FireExecuteChanged();
            }
        }
    }
}
