﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using RoomManagement.Web.Models;
using RoomManagement.Web.Services;
using System.ServiceModel.DomainServices.Client;

namespace RoomManagement.Views
{
    public partial class AddPersonWindow : ChildWindow, INotifyPropertyChanged
    {
        private readonly RoomContext _rc = new RoomContext();
        private IEnumerable<MailBox> _mailBoxes;
        private bool _areInputFieldsFilled;
        // when the visa is empty, both properties are false; otherwise, they have the expected meaning
        private bool _isVisaValid, _isVisaInvalid;

        public AddPersonWindow()
        {
            InitializeComponent();
            DataContext = this;
            MailBoxes = _rc.MailBoxes;
            _rc.Load( _rc.GetMailBoxesQuery() );
        }

        public IEnumerable<MailBox> MailBoxes
        {
            get { return _mailBoxes; }
            set
            {
                _mailBoxes = value;
                FirePropertyChanged( "MailBoxesList" );
            }

        }

        public bool IsPersonValid
        {
            get {
                return IsVisaValid && _areInputFieldsFilled;
            }
        }

        public bool IsVisaValid {
            get {
                return _isVisaValid;
            }
            set {
                _isVisaValid = value;
                FirePropertyChanged("IsVisaValid");
                FirePropertyChanged("IsPersonValid");
            }
        }

        public bool IsVisaInvalid {
            get {
                return _isVisaInvalid;
            }
            set {
                _isVisaInvalid = value;
                FirePropertyChanged("IsVisaInvalid");
            }
        }

        public Person AddedPerson = new Person();

        public static AddPersonWindow ShowDialog()
        {
            AddPersonWindow apw = new AddPersonWindow();
            apw.Show();
            return apw;
        }

        private void OKButton_Click( object sender, RoutedEventArgs e )
        {
            if ( string.IsNullOrWhiteSpace( TBfirstName.Text ) ||
               string.IsNullOrWhiteSpace( TBlastName.Text ) ||
               string.IsNullOrWhiteSpace( TBvisa.Text ) ||
               string.IsNullOrWhiteSpace( TBemail.Text ) ||
               CBmailBoxes.SelectedIndex == -1 )
            {
                SilverMessageBox.Show( "Please fill in all fields." );
                return;
            }
            AddedPerson.FirstName = TBfirstName.Text;
            AddedPerson.LastName = TBlastName.Text;
            AddedPerson.Visa = TBvisa.Text;
            AddedPerson.Email = TBemail.Text;
            AddedPerson.HomeMDB = ( (MailBox) CBmailBoxes.SelectedItem ).FullPath;
            AddedPerson.ID = Guid.NewGuid(); // This is a temp GUID. Since we can't change an AD user's objectGUID, it will be changed at next app load.
            AddedPerson.CN = TBfirstName.Text + " " + TBlastName.Text;
            AddedPerson.Flags = Constants.AdsUfNormalAccount | Constants.AdsDisabledAccount;
            DialogResult = true;
        }

        private void CancelButton_Click( object sender, RoutedEventArgs e )
        {
            DialogResult = false;
        }


        private void TBnames_LostFocus( object sender, RoutedEventArgs e )
        {
            if ( !string.IsNullOrWhiteSpace( TBfirstName.Text ) && !string.IsNullOrWhiteSpace( TBlastName.Text ) && string.IsNullOrWhiteSpace( TBemail.Text ) )
            {
                InvokeOperation<string> io = _rc.GetEmailSuffix();
                io.Completed += delegate
                {
                    TBemail.Text = ComputeEmailAddress(TBfirstName.Text, TBlastName.Text, io.Value);
                };
            }
            CheckInputFields();
        }

        private static string ComputeEmailAddress(string firstName, string lastName, string suffix) {

            // these replacement must be synchronized with ADHelper.cs, FixEmailAddress
            var replacements = new Dictionary<string, string> {
                { "ä", "ae"},
                { "â", "a"},
                { "à", "a"},
                { "é", "e"},
                { "è", "e"},
                { "ê", "e"},
                { "ë", "e"},
                { "ï", "i"},
                { "ì", "i"},
                { "î", "i"},
                { "ö", "oe"},
                { "ò", "o"},
                { "ó", "o"},
                { "ô", "o"},
                { "ü", "ue"},
                { "ù", "u"},
                { "ú", "u"},
                { "û", "u"},
                { "ñ", "n"},
                { "ç", "c"},
                { "`", "" },
                { "'", "" },
                { " ", "."}
            };

            StringBuilder sb = new StringBuilder(firstName.Trim().ToLowerInvariant() + "." + lastName.Trim().ToLowerInvariant());

            foreach (var kvp in replacements) {

                sb.Replace(kvp.Key, kvp.Value);
            }
            sb.Append(suffix);

            return sb.ToString();
        }

        private void CheckInputFields() {

            _areInputFieldsFilled = ! String.IsNullOrWhiteSpace(TBfirstName.Text) &&
                                    ! String.IsNullOrWhiteSpace(TBlastName.Text) &&
                                    ! String.IsNullOrWhiteSpace(TBemail.Text) &&
                                    CBmailBoxes.SelectedItem != null;
            FirePropertyChanged( "IsPersonValid" );
        }

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        private void FirePropertyChanged( string propertyName )
        {
            if ( PropertyChanged != null )
            {
                PropertyChanged( this, new PropertyChangedEventArgs( propertyName ) );
            }
        }
        #endregion

        private void OnVisaChanged(object sender, TextChangedEventArgs e) {

            CheckVisa(TBvisa.Text);
        }

        private void CheckVisa(string newVisa) {

            if (newVisa.Length < 3) {
                IsVisaValid = false;
                IsVisaInvalid = false;
            } else {
                _rc.IsNewVisaValid(newVisa,
                                   io => {
                                       IsVisaValid = io.Value;
                                       IsVisaInvalid = !IsVisaValid;
                                   }, null);
            }
        }

        private void OnMailBoxChanged(object sender, SelectionChangedEventArgs e) {
            CheckInputFields();
        }

        private void OnFirstNameChanged(object sender, TextChangedEventArgs e) {
            CheckInputFields();
        }

        private void OnLastNameChanged(object sender, TextChangedEventArgs e) {
            CheckInputFields();
        }

        private void OnEmailChanged(object sender, TextChangedEventArgs e) {
            CheckInputFields();
        }

    }
}