﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.ServiceModel.DomainServices.Client;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using RoomManagement.Web.Models;
using RoomManagement.Web.Services;

namespace RoomManagement.Views
{
    public partial class ReportView : Page, INotifyPropertyChanged, IRefreshable, ICanBeBusy
    {
        private readonly RoomContext _rc = new RoomContext();
        private EntitySet<Site> _sites;
        public EntitySet<Site> Sites
        {
            get { return _sites; }
            set
            {
                _sites = value;
                FirePropertyChanged( "Sites" );
            }
        }
        public ObservableCollection<Entity> FloorsAndTheirSites
        {
            get
            {
                ObservableCollection<Entity> entities = new ObservableCollection<Entity>();
                foreach ( Site s in SelectedSites )
                {
                    foreach ( Floor f in s.Floors )
                    {
                        entities.Add( f );
                    }
                    if ( s.Floors.Count > 1 && SelectedSites.Count > 1 ) // don't show a XXX total line if there is only one floor, or one selected site since there's already a "Total" line
                    {
                        entities.Add( s );
                    }
                }
                return entities;
            }
        }
        private List<Site> _selectedSites;
        public List<Site> SelectedSites
        {
            get { return _selectedSites; }
            set
            {
                _selectedSites = value;
                FirePropertyChanged( "SelectedSites" );
            }
        }
        // HACK...it's adding itself to this, to be able to have a Total column
        private ObservableCollection<ReportView> _totals;
        public ObservableCollection<ReportView> Totals
        {
            get { return _totals; }
            set
            {
                _totals = value;
                FirePropertyChanged( "Totals" );
            }
        }
        public bool IsBusy { get { return ( _rc.IsLoading || _rc.IsSubmitting ); } }

        public event EventHandler IsBusyChanged;

        public ReportView()
        {
            InitializeComponent();
            DataContext = this;

            Totals = new ObservableCollection<ReportView> { this };

            _rc.PropertyChanged += OnContextPropertyChanged;

            _selectedSites = SitesSelector.SelectedSites;

            Sites = _rc.Sites;

            LoadSites( _rc.GetSitesQuery() );
            LoadCellTypes();
        }

        private void OnContextPropertyChanged( object sender, PropertyChangedEventArgs e )
        {
            if ( e.PropertyName == "IsLoading" || e.PropertyName == "IsSubmitting" )
            {
                //FirePropertyChanged( "IsBusy" );
                if ( IsBusyChanged != null )
                {
                    IsBusyChanged( this, EventArgs.Empty );
                }
            }
        }

        private void LoadSites( EntityQuery<Site> query )
        {
            _rc.Load( query, LoadBehavior.RefreshCurrent, OnLoadedSites, null );
        }
        private void LoadPeople( EntityQuery<Site> query )
        {
            _rc.Load( query, LoadBehavior.RefreshCurrent, OnLoadedPeople, null );
        }
        private void LoadCellTypes()
        {
            _rc.Load( _rc.GetCellTypesQuery(), LoadBehavior.RefreshCurrent, OnLoadedCellTypes, null );
        }
        private void OnLoadedCellTypes( LoadOperation e )
        {
            if ( e.HasError )
            {
                SilverMessageBox.Show( Constants.ErrorMessage, "OK", e.Error.ToString() );
                e.MarkErrorAsHandled();
                return;
            }
        }
        private void OnLoadedSites( LoadOperation e )
        {
            if ( e.HasError )
            {
                SilverMessageBox.Show( Constants.ErrorMessage, "OK", e.Error.ToString() );
                e.MarkErrorAsHandled();
                return;
            }
            _rc.Load( _rc.GetPeopleBypassingCacheQuery(), LoadBehavior.RefreshCurrent, OnLoadedPeople, null );
        }
        private void OnLoadedPeople( LoadOperation e )
        {
            if ( e.HasError )
            {
                SilverMessageBox.Show( Constants.ErrorMessage, "OK", e.Error.ToString() );
                e.MarkErrorAsHandled();
                return;
            }

            foreach ( Site s in _rc.Sites )
            {
                s.RaiseAllChanges();
            }

            CBselectAll.IsChecked = true;
        }

        public void RefreshData()
        {
            _rc.RefreshSettingsData();
            LoadPeople( _rc.GetSitesQuery() );
            LoadCellTypes();
        }

        private void SelectAllCheckBox_Checked( object sender, RoutedEventArgs e )
        {
            Binding b = new Binding( "Sites" );
            BindingOperations.SetBinding( DGreport, DataGrid.ItemsSourceProperty, b );
            SelectedSites = new List<Site>();
            foreach ( Site s in Sites )
            {
                SelectedSites.Add( s );
            }
            RaiseTotalChanges();
        }
        private void SelectAllCheckBox_Unchecked( object sender, RoutedEventArgs e )
        {
            SitesSelector.SelectedSites = new List<Site>();
            _selectedSites = SitesSelector.SelectedSites;
            Binding b = new Binding( "FloorsAndTheirSites" );
            BindingOperations.SetBinding( DGreport, DataGrid.ItemsSourceProperty, b );
            RaiseTotalChanges();
        }

        private void SitesSelector_SelectedSitesChanged( object sender, EventArgs e )
        {
            FirePropertyChanged( "FloorsAndTheirSites" );
            FirePropertyChanged( "SelectedSites" );
            RaiseTotalChanges();
        }

        private void ExportToFileButton_Click( object sender, RoutedEventArgs e )
        {
            SaveFileDialog dialog = new SaveFileDialog { DefaultExt = "csv", Filter = "CSV Files (*.csv)|*.csv", FilterIndex = 1 };
            if ( dialog.ShowDialog() == true )
            {
                using ( StreamWriter writer = new StreamWriter( dialog.OpenFile(), Encoding.UTF8 ) )
                {
                    writer.Write( CreateFile() );
                }
            }
        }
        private void ExportToClipboardButton_Click( object sender, RoutedEventArgs e )
        {
            Clipboard.SetText( CreateFile( end: "\"\t" ) ); // Excel likes tabs as separators
            SilverMessageBox.Show( "The grid's content has been successfully copied to the Clipboard." );
        }
        private string CreateFile( string begin = "\"", string end = "\";" )
        {   // default = CSV
            StringBuilder file = new StringBuilder();

            file.Append( begin + "Location" + end );
            file.Append( begin + "Total cells" + end );
            file.Append( begin + "Equipped cells" + end );
            file.Append( begin + "Used cells" + end );
            file.Append( begin + "Desk sharing cells" + end );
            file.Append( begin + "People using desk sharing" + end );
            file.Append( begin + "Visitor cells" + end );
            file.Append( begin + "Available cells" + end );
            file.Append( begin + "Cells needing equipment" + end );
            file.Append( begin + "Free cells total" + end );
            file.Append( Environment.NewLine );
            if ( (bool) CBselectAll.IsChecked )
            {
                foreach ( Site s in Sites )
                {
                    file.Append( begin + s.Location + end );
                    file.Append( begin + s.CellsCount + end );
                    file.Append( begin + s.EquippedCellsCount + end );
                    file.Append( begin + s.UsedCellsCount + end );
                    file.Append( begin + s.DeskSharingCellsCount + end );
                    file.Append( begin + s.PeopleInDeskSharingCellsCount + end );
                    file.Append( begin + s.VisitorCellsCount + end );
                    file.Append( begin + s.AvailableCellsCount + end );
                    file.Append( begin + s.CellsNeedingEquipmentCount + end );
                    file.Append( begin + s.TotalFreeCellsCount + end );
                    file.Append( Environment.NewLine );
                }
            }
            else
            {
                foreach ( Entity e in FloorsAndTheirSites )
                {
                    Site s = e as Site;
                    if ( s != null )
                    {
                        file.Append( begin + s.Location + " total" + end );
                        file.Append( begin + s.CellsCount + end );
                        file.Append( begin + s.EquippedCellsCount + end );
                        file.Append( begin + s.UsedCellsCount + end );
                        file.Append( begin + s.DeskSharingCellsCount + end );
                        file.Append( begin + s.PeopleInDeskSharingCellsCount + end );
                        file.Append( begin + s.VisitorCellsCount + end );
                        file.Append( begin + s.AvailableCellsCount + end );
                        file.Append( begin + s.CellsNeedingEquipmentCount + end );
                        file.Append( begin + s.TotalFreeCellsCount + end );
                        file.Append( Environment.NewLine );
                    }
                    else
                    {
                        Floor f = e as Floor;
                        file.Append( begin + f.Location + end );
                        file.Append( begin + f.CellsCount + end );
                        file.Append( begin + f.EquippedCellsCount + end );
                        file.Append( begin + f.UsedCellsCount + end );
                        file.Append( begin + f.DeskSharingCellsCount + end );
                        file.Append( begin + f.PeopleInDeskSharingCellsCount + end );
                        file.Append( begin + f.VisitorCellsCount + end );
                        file.Append( begin + f.AvailableCellsCount + end );
                        file.Append( begin + f.CellsNeedingEquipmentCount + end );
                        file.Append( begin + f.TotalFreeCellsCount + end );
                        file.Append( Environment.NewLine );
                    }
                }
            }
            file.Append( begin + Total_Location + end );
            file.Append( begin + Total_CellsCount + end );
            file.Append( begin + Total_EquippedCellsCount + end );
            file.Append( begin + Total_UsedCellsCount + end );
            file.Append( begin + Total_DeskSharingCellsCount + end );
            file.Append( begin + Total_PeopleInDeskSharingCellsCount + end );
            file.Append( begin + Total_VisitorCellsCount + end );
            file.Append( begin + Total_AvailableCellsCount + end );
            file.Append( begin + Total_CellsNeedingEquipmentCount + end );
            file.Append( begin + Total_TotalFreeCellsCount + end );
            return file.ToString();
        }

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        private void FirePropertyChanged( string propertyName )
        {
            if ( PropertyChanged != null )
            {
                PropertyChanged( this, new PropertyChangedEventArgs( propertyName ) );
            }
        }
        #endregion

        #region Total
        public static string Total_Location
        {
            get
            {
                return "Total";
            }
        }
        public int Total_CellsCount
        {
            get
            {
                return SelectedSites.Sum(s => s.CellsCount);
            }
        }
        public int Total_TotalFreeCellsCount
        {
            get
            {
                return SelectedSites.Sum(s => s.TotalFreeCellsCount);
            }
        }
        public int Total_UsedCellsCount
        {
            get
            {
                return SelectedSites.Sum(s => s.UsedCellsCount);
            }
        }
        public int Total_EquippedCellsCount
        {
            get
            {
                return SelectedSites.Sum(s => s.EquippedCellsCount);
            }
        }
        public int Total_CellsNeedingEquipmentCount
        {
            get
            {
                return SelectedSites.Sum(s => s.CellsNeedingEquipmentCount);
            }
        }
        public int Total_DeskSharingCellsCount
        {
            get
            {
                return SelectedSites.Sum(s => s.DeskSharingCellsCount);
            }
        }
        public int Total_PeopleInDeskSharingCellsCount
        {
            get
            {
                return SelectedSites.Sum(s => s.PeopleInDeskSharingCellsCount);
            }
        }
        public int Total_VisitorCellsCount
        {
            get
            {
                return SelectedSites.Sum(s => s.VisitorCellsCount);
            }
        }
        public int Total_AvailableCellsCount
        {
            get
            {
                return SelectedSites.Sum(s => s.AvailableCellsCount);
            }
        }

        public void RaiseTotalChanges()
        {
            FirePropertyChanged( "Total_CellsCount" );
            FirePropertyChanged( "Total_TotalFreeCellsCount" );
            FirePropertyChanged( "Total_UsedCellsCount" );
            FirePropertyChanged( "Total_EquippedCellsCount" );
            FirePropertyChanged( "Total_CellsNeedingEquipmentCount" );
            FirePropertyChanged( "Total_DeskSharingCellsCount" );
            FirePropertyChanged( "Total_PeopleInDeskSharingCellsCount" );
            FirePropertyChanged( "Total_VisitorCellsCount" );
            FirePropertyChanged( "Total_AvailableCellsCount" );
            FirePropertyChanged( "SelectedSites" );
        }
        #endregion

        private void OnMainGridResized(object sender, EventArgs e) {

            CopyColumnWidths(DGreport, DGTotal);
        }

        private static void CopyColumnWidths(DataGrid dgSource, DataGrid dgDestination) {

            for (int i = 0; i < dgSource.Columns.Count; i++) {

                dgDestination.Columns[i].Width = new DataGridLength(dgSource.Columns[i].ActualWidth);
            }
        }
    }
}