﻿using System;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.ServiceModel.DomainServices.Client;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using RoomManagement.Web.Models;
using RoomManagement.Web.Services;

namespace RoomManagement.Views
{
    public partial class RoomsManagement : Page, INotifyPropertyChanged, IRefreshable, ICanBeBusy, IWantFocusNotification
    {
        private readonly RoomContext _rc = new RoomContext();

        private EntitySet<Site> _sites;
        public EntitySet<Site> Sites
        {
            get { return _sites; }
            set
            {
                _sites = value;
                FirePropertyChanged( "Sites" );
            }
        }
        private EntitySet<RoomType> _roomTypes;
        public EntitySet<RoomType> RoomTypes
        {
            get { return _roomTypes; }
            set
            {
                _roomTypes = value;
                FirePropertyChanged( "RoomTypes" );
            }
        }
        private EntitySet<CellType> _cellTypes;
        public EntitySet<CellType> CellTypes
        {
            get { return _cellTypes; }
            set
            {
                _cellTypes = value;
                FirePropertyChanged( "CellTypes" );
            }
        }
        public bool HasChanges { get { return ( _rc.HasChanges ); } }
        public bool IsBusy { get { return _rc.IsLoading || _rc.IsSubmitting; } }

        public event EventHandler IsBusyChanged;

        public RoomsManagement() {

            StateHelper.IsReady = false;
            StateHelper.Context = _rc;

            InitializeComponent();
            DataContext = this;

            _rc.PropertyChanged += OnContextPropertyChanged;

            Sites = _rc.Sites;
            CellTypes = _rc.CellTypes;
            RoomTypes = _rc.RoomTypes;

            RefreshData();
        }

        private void OnContextPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if ( e.PropertyName == "IsLoading" || e.PropertyName == "IsSubmitting" )
            {
                if ( IsBusyChanged != null )
                {
                    IsBusyChanged( this, EventArgs.Empty );
                }
            }
            if ( e.PropertyName == "HasChanges" )
            {
                FirePropertyChanged( "HasChanges" );
            }
        }

        private void LoadSites( EntityQuery<Site> query )
        {
            _rc.Load( query, LoadBehavior.RefreshCurrent, OnLoadedSites, null );
        }
        private void LoadCellTypes()
        {
            _rc.Load( _rc.GetCellTypesQuery(), LoadBehavior.RefreshCurrent, OnLoadedTypes, null );
        }
        private void LoadRoomTypes()
        {
            _rc.Load( _rc.GetRoomTypesQuery(), LoadBehavior.RefreshCurrent, OnLoadedTypes, null );
        }
        private void OnLoadedTypes( LoadOperation e )
        {
            if ( e.HasError )
            {
                SilverMessageBox.Show( Constants.ErrorMessage, "OK", e.Error.ToString() );
                e.MarkErrorAsHandled();
                return;
            }
        }
        private void OnLoadedSites( LoadOperation e )
        {
            if ( e.HasError )
            {
                SilverMessageBox.Show( Constants.ErrorMessage, "OK", e.Error.ToString() );
                e.MarkErrorAsHandled();
                return;
            }
            _rc.Load( _rc.GetPeopleQuery(), LoadBehavior.RefreshCurrent, OnLoadedPeople, null );
        }
        private void OnLoadedPeople( LoadOperation e )
        {
            if ( e.HasError )
            {
                SilverMessageBox.Show( Constants.ErrorMessage, "OK", e.Error.ToString() );
                e.MarkErrorAsHandled();
                return;
            }

            foreach ( Site s in _rc.Sites )
            {
                s.RaiseAllChanges();
            }
            StateHelper.IsReady = true;
        }

        private void AddSiteButton_Click( object sender, RoutedEventArgs e )
        {
            Site s = new Site
            {
                Name = "Name",
                Location = "Fill me in !"
            };
            Sites.Add( s );
        }
        private void AddFloorButton_Click( object sender, RoutedEventArgs e )
        {
            Site selectedSite = (Site) DGsites.SelectedItem;
            if ( selectedSite == null ) return;
            Floor f = new Floor
            {
                Number = 0000,
                Location = selectedSite.Location + ", somewhere"
            };
            selectedSite.Floors.Add( f );
        }
        private void AddRoomButton_Click( object sender, RoutedEventArgs e )
        {
            Floor selectedFloor = (Floor) DGfloors.SelectedItem;
            if ( selectedFloor == null ) return;
            int number = selectedFloor.Rooms.Count + 1 + ( selectedFloor.Number * 100 );
            Room r = new Room
             {
                 Number = number,
                 RoomType = null,
                 PhoneNumber = string.Empty,
                 IsSecured = false,
                 PersonGroup = string.Empty
             };
            selectedFloor.Rooms.Add( r );
        }
        private void AddCellButton_Click( object sender, RoutedEventArgs e )
        {
            Room selectedRoom = (Room) DGrooms.SelectedItem;
            if ( selectedRoom == null ) return;
            byte number = (byte) ( selectedRoom.Cells.Count + 1 );
            CellType type = selectedRoom.Cells.Count > 0 ? selectedRoom.Cells.ElementAt( 0 ).CellType : CellTypes.ElementAt( 0 );
            Cell c = new Cell
            {
                Number = number,
                CellType = type,
                IsEquipped = false,
            };
            selectedRoom.Cells.Add( c );
        }
        private void AddRoomTypeButton_Click( object sender, RoutedEventArgs e )
        {
            RoomType rt = new RoomType
            {
                Name = "Fill me in !",
                Description = "Fill me in !"
            };
            RoomTypes.Add( rt );
        }
        private void AddCellTypeButton_Click( object sender, RoutedEventArgs e )
        {
            CellType ct = new CellType
            {
                Name = "Fill me in !",
                Description = "Fill me in !"
            };
            CellTypes.Add( ct );
        }

        private void DeleteSiteButton_Click( object sender, RoutedEventArgs e )
        {
            if ( DGsites.SelectedItem == null ) return;
            SilverMessageBox smb = SilverMessageBox.Show( "Are you REALLY sure you want to do that? This site's floors, rooms and cells will be deleted.", "Yes", button2Text: "No" );
            smb.Closed += delegate
            {
                if ( smb.DialogResult == true ) // Yes
                {
                    _rc.Sites.Remove( (Site) DGsites.SelectedItem );
                }
                else return;
            };
        }
        private void DeleteFloorButton_Click( object sender, RoutedEventArgs e )
        {
            if ( DGfloors.SelectedItem == null ) return;
            SilverMessageBox smb = SilverMessageBox.Show( "Are you REALLY sure you want to do that? This floor's rooms and cells will be deleted.", "Yes", button2Text: "No" );
            smb.Closed += delegate
            {
                if ( smb.DialogResult == true ) // Yes
                {
                    _rc.Floors.Remove( (Floor) DGfloors.SelectedItem );
                }
                else return;
            };
        }
        private void DeleteRoomButton_Click( object sender, RoutedEventArgs e )
        {
            if ( DGrooms.SelectedItem == null ) return;
            SilverMessageBox smb = SilverMessageBox.Show( "Are you REALLY sure you want to do that? This room's cells will be deleted.", "Yes", button2Text: "No" );
            smb.Closed += delegate
            {
                if ( smb.DialogResult == true ) // Yes
                {
                    _rc.Rooms.Remove( (Room) DGrooms.SelectedItem );
                }
                else return;
            };
        }
        private void DeleteCellButton_Click( object sender, RoutedEventArgs e )
        {
            if ( DGcells.SelectedItem == null ) return;
            SilverMessageBox smb = SilverMessageBox.Show( "Are you sure you want to do that? This cell will be deleted.", "Yes", button2Text: "No" );
            smb.Closed += delegate
            {
                if ( smb.DialogResult == true ) // Yes
                {
                    _rc.Cells.Remove( (Cell) DGcells.SelectedItem );
                }
                else return;
            };
        }
        private void DeleteRoomTypeButton_Click( object sender, RoutedEventArgs e )
        {
            if ( DGroomTypes.SelectedItem == null ) return;
            SilverMessageBox smb = SilverMessageBox.Show( "Are you sure you want to do that? This room type will be deleted.", "Yes", button2Text: "No" );
            smb.Closed += delegate
            {
                if ( smb.DialogResult == true ) // Yes
                {
                    RoomTypes.Remove( (RoomType) DGroomTypes.SelectedItem );
                }
                else return;
            };
        }
        private void DeleteCellTypeButton_Click( object sender, RoutedEventArgs e )
        {
            if ( DGcellTypes.SelectedItem == null ) return;
            SilverMessageBox smb = SilverMessageBox.Show( "Are you sure you want to do that? This cell type will be deleted.", "Yes", button2Text: "No" );
            smb.Closed += delegate
            {
                if ( smb.DialogResult == true ) // Yes
                {
                    CellTypes.Remove( (CellType) DGcellTypes.SelectedItem );
                }
                else return;
            };
        }

        private void SaveButton_Click( object sender, RoutedEventArgs e )
        {
            SaveButton.IsEnabled = false;
            // SubmitChanges will merge data from the server to the client (here). 
            // As Site.DisplayedPhoneRanges is unknown at the server side, it will be set to null by RIA Services =>
            // Tell Site to ignore the future call to Site.DisplayedPhoneRanges = null
            StateHelper.IsReady = false;
            _rc.SubmitChanges( delegate(SubmitOperation so) { 
                                    SaveButton.IsEnabled = true;
                                    StateHelper.IsReady = true;
                                    if (so.HasError) {
                                        StringBuilder sb = new StringBuilder("Error while saving changes to room data:" + Environment.NewLine);
                                        foreach (Entity en in so.EntitiesInError) {
                                            if (en.HasValidationErrors) {
                                                foreach (ValidationResult vri in en.ValidationErrors) {
                                                    sb.AppendLine(vri.ErrorMessage);
                                                }
                                            }
                                        }
                                        SilverMessageBox.Show(sb.ToString());
                                        so.MarkErrorAsHandled();
                                    }
                               }, null );
        }
        private void RejectButton_Click( object sender, RoutedEventArgs e )
        {
            SilverMessageBox smb = SilverMessageBox.Show( "Do you really want to reject all the changes you just made ?", "Yes", button2Text: "No" );
            smb.Closed += delegate
            {
                if ( smb.DialogResult == true ) // Yes
                {
                    _rc.RejectChanges();
                }
                else return;
            };
        }

        public void RefreshData()
        {
            _rc.RefreshSettingsData();
            LoadSites( _rc.GetAllSitesQuery() );
            LoadCellTypes();
            LoadRoomTypes();
        }

        private void DGrooms_LoadingRow( object sender, DataGridRowEventArgs e ) {
            DataGridHelper.SetItemsSource(DGrooms, e.Row, RoomTypes);
        }

        private void DGcells_LoadingRow( object sender, DataGridRowEventArgs e ) {
            DataGridHelper.SetItemsSource(DGcells, e.Row, CellTypes);
        }

        private void DGsites_LoadingRow(object sender, DataGridRowEventArgs e) {

            DataGridHelper.SetItemsSource(DGsites, e.Row, Site.RpisRules);
        }

        private void CBcellType_SelectionChanged( object sender, SelectionChangedEventArgs e )
        {
            ComboBox cb = (ComboBox) sender;
            Cell c = (Cell) cb.DataContext;
            CellType type = cb.SelectedItem as CellType;
            if ( type != null && c.People.Count > 1 && type.ID != Constants.DeskSharingCellTypeId )
            {
                SilverMessageBox.Show( "This cell has more than one person in it. It can only be a desk sharing place." );
                cb.SelectedItem = CellTypes.Where( ct => ct.ID == Constants.DeskSharingCellTypeId ).First();
            }
        }

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        private void FirePropertyChanged( string propertyName )
        {
            if ( PropertyChanged != null )
            {
                PropertyChanged( this, new PropertyChangedEventArgs( propertyName ) );
            }
        }
        #endregion

        private void OnSiteSelected(object sender, SelectionChangedEventArgs e) {

            Site s = (Site)DGsites.SelectedItem;

            if (s != null && s.Floors.Count == 1) {

                DGfloors.SelectedIndex = 0;
            }
        }

        private void OnFloorSelected(object sender, SelectionChangedEventArgs e) {

            Floor f = (Floor)DGfloors.SelectedItem;

            if (f != null && f.Rooms.Count == 1) {
                DGrooms.SelectedIndex = 0;
            }
        }

        private void OnRoomSelected(object sender, SelectionChangedEventArgs e) {

            Room r = (Room)DGrooms.SelectedItem;

            if (r != null && r.Cells.Count == 1) {

                DGcells.SelectedItem = 0;
            }
        }

        public void HasLostFocus() {
            StateHelper.IsReady = false;
        }

        public void HasGotFocus() {
            // not really interested
        }
    }
}